//hlavni funkce modulu
//
//Typy
//Loktor
type TLocator = Record
	Locator	:	String;
	Count	:	Integer;
end;
//Definice zem
type TCountryDef = Record
	Country : String;	//Zem (groundspeak nzev)
	SQLrow: String;		//Filtr tverc - dky (slo dku,SQL GLOB filtr|...)
	SQLcol:	String;		//Filtr tverc - sloupce (slo sloupce,SQL GLOB filtr|...)
	Info : String;		//Popis tverc (x,y,text|...)
	xm : integer;			//poet verc x*24
	ym : integer;			//poet tverc y*24
	//flag: String;			//vlajka (z geocaching.com)
end;

//Globln konstanty
const
	Version = '1.0.0';
//Globln promnn
var
	MapLoc : array of array of TLocator;

procedure QTH_ParseDelimited(const sl : TStringList; const value : string; const delimiter : string);
var
   dx : integer;
   ns : string;
   txt : string;
   delta : integer;
begin
   delta := Length(delimiter) ;
   txt := value + delimiter;
   sl.BeginUpdate;
   sl.Clear;
   try
     while Length(txt) > 0 do
     begin
       dx := Pos(delimiter, txt) ;
       ns := Copy(txt,0,dx-1) ;
       if trim(ns) <> '' then sl.Add(ns) ;
       txt := Copy(txt,dx+delta,65536) ;
     end;
   finally
     sl.EndUpdate;
   end;
end;

//-------------------------------------------------------------------------
function StrFloatShort(fl:String;d:integer):String;
begin
	Result := SeparateLeft(fl,'.')+'.'+Copy(SeparateRight(fl,'.'),1,d);
end;

function GetQTHStateData(sCountry:String):String;
var ini:TMemIniFile;
    s:string;
begin
  //nacte states definici QTH mapy podle jmena zeme
  Result:='';
  ini:=TMemIniFile.Create(GetModulesDir()+'QTHMap.MapDefinitions.ini');
  try
	s:=ini.ReadString('QTHMapDefinitions',sCountry,'');
    Result:=s;
  finally
    ini.Free();
  end;
end;


function getSQLpart(cd:TCountryDef;name:String):string;
var
	sSQL,sData,sPart,sPozice,sSablona:String;
	i,j: integer;
begin
	sSablona := ' CASE WHEN substr(gv.value,1,4) GLOB "%1%" THEN %2% ELSE';
	sSQL := ''; i := 0;
	case name of
	'rw': sData := cd.SQLrow;
	'cl': sData := cd.SQLcol;
	end;
	//{$ifdef DEBUG_HELPER}
	//LDH('Data:'+ sData);
	//{$endif}
	while Length(sData) <> 0 do begin
		sPart := Fetch(sData,'|');					//Ukusuje etzec po |
		sPozice := Fetch(sPart,',');				//Pozice a zbytek je Filtr
		//
		sSQL := sSQL + StringOfChar(' ',i) + ReplaceString(ReplaceString(sSablona,'%2%',sPozice), '%1%', sPart);
		Inc(i);
	end;
	sSQL := sSQL + ' -1';
	for j := i-1 downto 0 do begin
		sSQL := sSQL + StringOfChar(' ',j) + ' END';
	end;
	Result := sSQL + ' ' + name + ',';
end;

function getGraphic(cd:TCountryDef;We,He,Wd:integer):string;
var 
	sGraphic,sData,sCtverec :String;
	x,y,i:integer;
begin
	sGraphic := '';
	sData := cd.Info; i:=0;
	while Length(sData) <> 0 do begin
		sCtverec := Fetch(sData,'|');					//Ukusuje info etzec
		x := StrToInt(Fetch(sCtverec,','));		//x
		y := StrToInt(Fetch(sCtverec,','));		//y
		//
		if i > 0 then sGraphic := sGraphic +','+CRLF;
		//Popis obdlnk
		sGraphic := sGraphic + '    {'+CRLF
    	+ '      type: ''text'','+CRLF
		+ '      left: '''+IntToStr(Wd+We*x+25)+'px'','+CRLF
		+ '      top: '''+IntToStr(He*y+11)+'px'','+CRLF
    	+ '      z: 0,'+CRLF
    	+ '      style: {'+CRLF
    	+ '        fill: ''#333'','+CRLF
		+ '        text: ['+CRLF
    	+ '               '''+sCtverec+''''+CRLF
    	+ '        ].join(''\n''),'+CRLF
    	+ '        font: ''16px Microsoft YaHei'''+CRLF
    	+ '      }'+CRLF
    	+ '    }';
		//
		Inc(i);
		//
	end;
	sGraphic := sGraphic +CRLF;
	//
	Result := sGraphic;
end;
	
function QTHMap_Run(aPar: TStringList):string;
var 
	i,j,iW,iH,iWe,iHe,iWd:integer;
	xmax,ymax,x,y,cmax,sum,lsum,cnt:integer;
	sql,country,s: String;
	table: TSQLiteTable;					//Tabulka
	sEOption,sdata,sEId,sEIdold,sGraph: String;
	CDef: TCountryDef;
		
begin
	Result:='';
	//
	if(aPar.Values['Title']<>'') then Result:=ProcessCommand('CmdTitle',aPar);
	//
	country := aPar.Values['Country']
	s := GetQTHStateData(country);
	//
	CDef.Country := country;
	CDef.xm := StrToInt(Fetch(s,';'));
	CDef.ym := StrToInt(Fetch(s,';'));
	CDef.SQLrow := Fetch(s,';');
	CDef.SQLcol := Fetch(s,';');
	CDef.Info := s;

	{$ifdef DEBUG_HELPER}
	LDH('Country:'+ country);
	{$endif}
	//GetArraySize(country,xmax,ymax);
	xmax := CDef.xm; ymax := CDef.ym;
	if xmax + ymax < 0 then exit;
	//Dynamick pole X/Y------------
	//{$ifdef DEBUG_HELPER}
	//LDH('Xmax/Ymax:'+ IntToStr(Xmax)+'/'+IntToStr(Ymax));
	//{$endif}
	//Inicializace------------------
	SetLength(MapLoc,ymax);
	For i:=0 to ymax-1 do begin
		SetLength(MapLoc[i],xmax);
	end;
	//------------------------------
	for i:=0 to ymax-1 do begin
		for j:=0 to xmax-1 do begin
			MapLoc[i][j].Locator := '';
			MapLoc[i][j].Count := 0;
		end;
	end;
	//------------------------------
	cmax := 0; sum := 0; lsum := 0; cnt := 0; sdata:='';
	//--------
	sql:='WITH tmp(locator,rw,cl,loccnt,loccountry) AS (SELECT';
  	sql:=sql+' gv.value AS locator,';
	//Podmnky pro sloupce a dky SQL pro kadou zemi jsou extra jin (idx)
	sql:=sql+getSQLpart(CDef,'rw');	//dky
	sql:=sql+getSQLpart(CDef,'cl');	//sloupce
	//-----------------------------------------------------------------------
  	sql:=sql+' COUNT(gc.id) loccnt,';
  	sql:=sql+' gc.country loccountry';
	sql:=sql+' FROM geocache gc';
	sql:=sql+' INNER JOIN geotag gt ON gt.id = gc.id';
	sql:=sql+' INNER JOIN geotagcategory gtc ON gtc.key = gt.ptrkat AND gtc.value = "QTHLocator"';
	sql:=sql+' INNER JOIN geotagvalue gv ON gv.key = gt.ptrvalue';
	sql:=sql+' WHERE gc.dtfound > 0 AND gc.country = "'+country+'"';
  	sql:=sql+' GROUP BY gv.value, gc.country)';
	sql:=sql+' '+CRLF;
	sql:=sql+' SELECT';
  	sql:=sql+'  x.locator,';
  	sql:=sql+'  x.rw rw,';
  	sql:=sql+'  x.cl cl,';
  	sql:=sql+'  x.cl*24+(unicode(substr(x.locator,5,1))-65) AS xx,';
  	sql:=sql+'  x.rw*24+(23-(unicode(substr(x.locator,6,1))-65)) AS yy,';
  	sql:=sql+'  x.cnt cnt,';
  	sql:=sql+'  x.country country';
	sql:=sql+' FROM (SELECT';
  	sql:=sql+'  t.locator locator,';
	sql:=sql+'  MAX(t.rw) rw,';
	sql:=sql+'  MAX(t.cl) cl,';
	sql:=sql+'  SUM(t.loccnt) cnt,';
	sql:=sql+'  group_concat(t.loccountry) country';
	sql:=sql+' FROM tmp t';
	sql:=sql+' GROUP BY locator) x';
	sql:=sql+' ORDER BY xx,yy desc'; 
	//
	{$ifdef DEBUG_HELPER}
	LDH('SQL:' + sql);
	{$endif}
	//
	table := GEOGET_DB.GetTable(sql, false);
	//0-locator name, 1-column, 2-row, 3- xx, 4-yy, 5-caches, 6-country
	try
		while not table.Eof do begin
			Inc(cnt);
			x:=table.FieldAsInteger(3);	//xx
			y:=table.FieldAsInteger(4);	//yy
			//{$ifdef DEBUG_HELPER}
			//LDH('loc,X/Y:' + table.FieldAsString(0) +', '+ IntToStr(x)+'/'+IntToStr(y));
			//{$endif}
			if (x>=0) and (y>=0) then  //kontrola abnormlnch ke
			begin
				MapLoc[y][x].Locator := table.FieldAsString(0);	//Locator name
				MapLoc[y][x].Count := table.FieldAsInteger(5);	//Cahes count
				sum := sum + MapLoc[y][x].Count;								//Suma ke v dan zemi
				if MapLoc[y][x].Count > cmax then  cmax := MapLoc[y][x].Count;	//nejvy poet ke
				Inc(lsum);
			end;
			//------------
			table.Next;
		end;
	finally
		table.Free;
	end;
	//Data
	for i:=0 to ymax-1 do begin
		for j:=0 to xmax-1 do begin
			if MapLoc[i][j].Count > 0 then begin
				sdata := sdata + '['+IntToStr(i)+','+IntToStr(j)+','+IntToStr(MapLoc[i][j].Count)+','''+MapLoc[i][j].Locator+'''],'
			end;
		end;
	end;
	sdata := '['+sdata+']';
	sdata := ReplaceString(sdata,'],]',']]');
	//
	sEId := 'id'+IntToStr(iItem);
	//Graf
	iW := (iWidth/100)*96;	//96%
	if xmax > 72 then begin
		iWe := iW/(xmax/24);
		iWd := (iWidth-iWe*(xmax/24))/3
	end
	else begin
		iWe := iW/4;
		iWd := (iWidth-iWe*(xmax/24))/2;
	end;
	iH := ((iWe*(ymax/24))/100)*86;	//86%
	iHe := iH/(ymax/24);	
	//
	sEOption := 'var d'+sEId+' = '+sdata+';'+CRLF
	+ CRLF
	+ 'd = d'+sEId+'.map(function (item) {'+CRLF
  	+ '  return [item[1], item[0], item[2] || ''-''];'+CRLF
	+ '});'+CRLF
  	+ CRLF
  	+ 'option = {'+CRLF
  	+ '  tooltip: {'+CRLF
  	+ '    position: ''top'','+CRLF
	+ '    formatter: function(item,index) {'+CRLF
  	+ '      parts = index.split("_");'+CRLF
  	+ '      return "QTH: "+ d'+sEId+'[parts[2]][3]+"</br>'+_('Caches:')+' "+d'+sEId+'[parts[2]][2];'+CRLF
  	+ '    }'+CRLF
  	+ '  },'+CRLF
  	+ '  animation: true,'+CRLF
  	+ '  grid: {'+CRLF
  	+ '    height: '''+IntToStr(iH)+'px'','+CRLF
	+ '    width: '''+IntToStr(iWe*(xmax/24))+'px'','+CRLF
	+ '    x: '''+IntToStr(iWd+18)+'px'','+CRLF
  	+ '    y: ''5px'''+CRLF
  	+ '  },'+CRLF
  	+ '  xAxis: {'+CRLF
  	+ '    type: ''value'','+CRLF
  	+ '    position: ''top'','+CRLF
  	+ '    min: 0,'+CRLF
  	+ '    max: '+IntToStr(xmax)+','+CRLF
  	+ '    interval: 24,'+CRLF
  	+ '    axisLabel: {'+CRLF
  	+ '      show: false'+CRLF
  	+ '    },'+CRLF
  	+ '    splitLine: {'+CRLF
  	+ '      interval: 23,'+CRLF
  	+ '      show: true'+CRLF
  	+ '    },'+CRLF
  	+ '    splitArea: {'+CRLF
  	+ '      interval: 23,'+CRLF
  	+ '      show: true'+CRLF
  	+ '    }'+CRLF
  	+ '  },'+CRLF
  	+ '  yAxis: {'+CRLF
  	+ '    type: ''value'','+CRLF
  	+ '    inverse: true,'+CRLF
  	+ '    min: 0,'+CRLF
  	+ '    max: '+IntToStr(ymax)+','+CRLF
  	+ '    interval: 24,'+CRLF
  	+ '    axisLabel: {'+CRLF
  	+ '      show: false'+CRLF
  	+ '    },'+CRLF
  	+ '    splitLine: {'+CRLF
  	+ '      interval: 23,'+CRLF
  	+ '      show: true'+CRLF
  	+ '    },'+CRLF
  	+ '    splitArea: {'+CRLF
  	+ '      interval: 23,'+CRLF
  	+ '      show: true'+CRLF
  	+ '    }'+CRLF
  	+ '  },'+CRLF
	+ '  graphic: ['+CRLF
	sEOption := sEOption + getGraphic(CDef,iWe,iHe,iWd);	//idx u definuje zemi
	sEOption := sEOption
	+ '  ],'+CRLF
  	+ '  visualMap: {'+CRLF
  	+ '    min: 0,'+CRLF
  	+ '    max: '+IntToStr(cmax)+','+CRLF
  	+ '    calculable: true,'+CRLF
  	+ '    orient: ''horizontal'','+CRLF
  	+ '    left: ''center'','+CRLF
  	+ '    bottom: ''0%'','+CRLF
	+ '    inRange: {'+CRLF
  	+ '      color: [''#C86106'', ''#E6E600'' ]'+CRLF
  	+ '    }'+CRLF
  	+ '  },'+CRLF
  	+ '  series: [{'+CRLF
  	+ '    name: '''+ReplaceString(country,' ','')+''','+CRLF
  	+ '    type: ''heatmap'','+CRLF
  	+ '    data: d,'+CRLF
  	+ '    label: {'+CRLF
  	+ '      normal: {'+CRLF
  	+ '        show: false'+CRLF
  	+ '      }'+CRLF
  	+ '    },'+CRLF
  	+ '    itemStyle: {'+CRLF
  	+ '      emphasis: {'+CRLF
  	+ '        shadowBlur: 10,'+CRLF
  	+ '        shadowColor: ''rgba(0, 0, 0, 0.5)'''+CRLF
  	+ '      }'+CRLF
  	+ '    }'+CRLF
  	+ '  }]'+CRLF
  	+ '};'+CRLF;
	//-----------------------------
  	// finln sestaven URL grafu
	//ECharts --------------------
	sEIdold := sEId;
	sEId:='ID'+IntToStr(iItem) + 'gr'; //+ReplaceString(country,' ',''); //+'mol'+IntToStr(idx);
	//
	Result:=Result+'<p style="text-align:center;font-size:medium;"><span><img src="%IconCountry'+CDef.Country+'%"></span>&nbsp;<span>Maidenhead Locator System (QTH) - '
	+_('Occupied locators: ')+IntToStr(lsum)+'</span></p>';
	Output(Result);
	Result:='';
	// 
	sGraph:='<div id="'+sEId+'" style="width:'+IntToStr(iWidth)+'px; height:'+IntToStr(iHe*(ymax/24)+((iHe*3)/4))+'px;"></div>'+CRLF
	+'<script type="text/javascript">'+CRLF
	+'  var chart_'+sEId+'=echarts.init(document.getElementById('''+sEId+'''));'+CRLF
	+sEOption + CRLF
  	+ ' chart_'+sEId+'.on(''click'', function (params) {'+CRLF
	//+ '   console.log(params);'+CRLF
	+ '   if (params.info == "QTHeader") {'+CRLF
	+ '     sURL = ''https://www.geoget.cz/doku.php/user:skript:stator:modules:stator-qthmap'';'+CRLF
	+ '     window.open(sURL,''_blank'');'+CRLF
	+ '   }'+CRLF
	+ '   if (params.componentType == "series") {'+CRLF
  	+ '     sURL = ''http://www.cbpmr.cz/lokatorova-mapa-gps.html#l=''+d'+sEIdold+'[params.dataIndex][3];'+CRLF
  	+ '     window.open(sURL,''_blank'');'+CRLF
	+ ' }});'+CRLF
	//vytvoreni grafu s potrebnymi parametry a definovanym tematem
	+'  chart_'+sEId+'.setOption(option);'+CRLF
	+'</script>'+CRLF+CRLF;		 
	//vse uz je prevedeno do UTF a entit, neprevadet znovu
	//sGraph:=AnsiToUtf(sGraph);  // prevedeme na utf
	{$ifdef DEBUG_HELPER}
	LDH('GraphData:'+CRLF+sGraph);
	{$endif}
	//
	Result := Result + sGraph;
	OutputNoReplace(Result);
  	Result:='';
  	// klid
	//------------------------------
	//Mazn pol
	//------------------------------
	For i:=0 to ymax-1 do begin
		SetLength(MapLoc[i],0);
	end;
	SetLength(MapLoc,0);
	//------------------------------
end;